@@ -22,12 +22,21 @@ import os
 import tempfile
 
 import pandas as pd
-from pandas._config.config import (
-    get_option,
-    register_option,
-    reset_option,
-    set_option,
-)
+
+try:
+    from pandas._config.config import (
+        get_option,
+        register_option,
+        reset_option,
+        set_option,
+    )
+except ModuleNotFoundError:
+    from pandas.core.config import (
+        get_option,
+        register_option,
+        reset_option,
+        set_option,
+    )
 
 options = pd.options
 
@@ -39,3 +48,7 @@ register_option(CONF_RIKAI_CACHEURI, DEFAULT_RIKAI_CACHEURI)
 CONF_PARQUET_BLOCK_SIZE = "parquet.block.size"
 DEFAULT_ROW_GROUP_SIZE_BYTES = 32 * 1024 * 1024
 register_option(CONF_PARQUET_BLOCK_SIZE, DEFAULT_ROW_GROUP_SIZE_BYTES)
+
+CONF_RIKAI_IMAGE_DEFAULT_FORMAT = "rikai.image.default.format"
+DEFAULT_IMAGE_DEFAULT_FORMAT = "PNG"
+register_option(CONF_RIKAI_IMAGE_DEFAULT_FORMAT, DEFAULT_IMAGE_DEFAULT_FORMAT)
