@@ -16,6 +16,7 @@ import re
 import rikai
 from rikai.__version__ import version
 from rikai.conf import CONF_PARQUET_BLOCK_SIZE
+from rikai.spark.sql import init
 
 
 def df_to_rikai(df: "pyspark.sql.DataFrame", uri: str):
@@ -43,3 +44,34 @@ def get_default_jar_version(use_snapshot=True):
     if use_snapshot and (len(match_str) < len(version)):
         return match_str + "-SNAPSHOT"
     return match_str
+
+
+def init_spark_session(conf=None, app_name="rikai", rikai_version=None):
+    from pyspark.sql import SparkSession
+
+    if not rikai_version:
+        rikai_version = get_default_jar_version(use_snapshot=True)
+    builder = (
+        SparkSession.builder.appName(app_name)
+        .config(
+            "spark.jars.packages", "ai.eto:rikai_2.12:{}".format(rikai_version)
+        )
+        .config(
+            "spark.sql.extensions",
+            "ai.eto.rikai.sql.spark.RikaiSparkSessionExtensions",
+        )
+        .config(
+            "spark.driver.extraJavaOptions",
+            "-Dio.netty.tryReflectionSetAccessible=true",
+        )
+        .config(
+            "spark.executor.extraJavaOptions",
+            "-Dio.netty.tryReflectionSetAccessible=true",
+        )
+    )
+    conf = conf or {}
+    for k, v in conf.items():
+        builder = builder.config(k, v)
+    session = builder.master("local[2]").getOrCreate()
+    init(session)
+    return session
